@@ -1,18 +1,15 @@
 BUILD_BIT='32bit'
 BUILD_CPU='ARMv7 Processor rev 3 (v7l)'
-BUILD_DATE='2019-12-11'
+BUILD_DATE='2019-12-19'
 BUILD_MACHINE='armv7l'
 BUILD_OS='Linux unknown unknown unknown'
-BUILD_TIME='13:49'
+BUILD_TIME='22:29'
 COMPILER_VERSION='armv6l-unknown-linux-gnueabihf-gcc (GCC) 8.3.0'
 CYTHON_VERSION='0.29.14'
 LINKER_VERSION='GNU ld (GNU Binutils) 2.32'
 PYTHON_VERSION='3.8.0'
 RELEASE_BUILD=True
 lib_gtk='3.24.13'
-lib_pycairo='1.18.2'
-lib_pygobject='2.28.7'
-lib_pygtk='2.24.0'
 lib_vpx='1.8.1'
 lib_x11='1.6.9'
 lib_x264='0.157.2980'
