@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2013-2018 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2018 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -11,59 +11,46 @@ import sys
 import signal
 import uuid
 import time
+import struct
 import binascii
 
 #hide some ugly python3 compat:
 try:
     import _thread as thread            #@UnresolvedImport @UnusedImport (python3)
-except:
+except ImportError:
     import thread                       #@Reimport @UnusedImport
+assert thread
 
 try:
     from queue import Queue             #@UnresolvedImport @UnusedImport (python3)
 except ImportError:
     from Queue import Queue             #@Reimport @UnusedImport
+assert Queue
 
 try:
     import builtins                     #@UnresolvedImport @UnusedImport (python3)
-except:
+except ImportError:
     import __builtin__ as builtins      #@Reimport @UnusedImport
+assert builtins
 _buffer = builtins.__dict__.get("buffer")
 
 
 SIGNAMES = {}
-for x in [x for x in dir(signal) if x.startswith("SIG") and not x.startswith("SIG_")]:
-    SIGNAMES[getattr(signal, x)] = x
-
-
-#use cStringIO, fallback to StringIO,
-#and python3 is making life more difficult yet again:
-try:
-    from io import BytesIO as BytesIOClass              #@UnusedImport
-except:
-    try:
-        from cStringIO import StringIO as BytesIOClass  #@Reimport @UnusedImport
-    except:
-        from StringIO import StringIO as BytesIOClass   #@Reimport @UnusedImport
-try:
-    from StringIO import StringIO as StringIOClass      #@UnusedImport
-except:
-    try:
-        from cStringIO import StringIO as StringIOClass #@Reimport @UnusedImport
-    except:
-        from io import StringIO as StringIOClass        #@Reimport @UnusedImport
+for signame in (sig for sig in dir(signal) if sig.startswith("SIG") and not sig.startswith("SIG_")):
+    SIGNAMES[getattr(signal, signame)] = signame
 
 
 WIN32 = sys.platform.startswith("win")
 OSX = sys.platform.startswith("darwin")
 LINUX = sys.platform.startswith("linux")
+NETBSD = sys.platform.startswith("netbsd")
+OPENBSD = sys.platform.startswith("openbsd")
 FREEBSD  = sys.platform.startswith("freebsd")
 
 POSIX = os.name=="posix"
 PYTHON2 = sys.version_info[0]==2
 PYTHON3 = sys.version_info[0]==3
 
-import struct
 BITS = struct.calcsize(b"P")*8
 
 
@@ -76,11 +63,11 @@ if PYTHON2:
         return binascii.hexlify(str(v))
 else:
     def strtobytes(x):
-        if type(x)==bytes:
+        if isinstance(x, bytes):
             return x
         return str(x).encode("latin1")
     def bytestostr(x):
-        if type(x)==bytes:
+        if isinstance(x, bytes):
             return x.decode("latin1")
         return str(x)
     def hexstr(v):
@@ -96,7 +83,7 @@ def get_util_logger():
     return util_logger
 
 def memoryview_to_bytes(v):
-    if type(v)==bytes:
+    if isinstance(v, bytes):
         return v
     if isinstance(v, memoryview):
         return v.tobytes()
@@ -161,7 +148,7 @@ def get_group_id(group):
         import grp      #@UnresolvedImport
         gr = grp.getgrnam(group)
         return gr.gr_gid
-    except:
+    except (ImportError, KeyError):
         return -1
 
 
@@ -171,7 +158,7 @@ def platform_release(release):
         try:
             import plistlib
             if PYTHON2:
-                pl = plistlib.readPlist('/System/Library/CoreServices/SystemVersion.plist')
+                pl = plistlib.readPlist('/System/Library/CoreServices/SystemVersion.plist') #pylint: disable=deprecated-method
             else:
                 with open(SYSTEMVERSION_PLIST, "rb") as f:
                     pl = plistlib.load(f)           #@UndefinedVariable
@@ -184,7 +171,7 @@ def platform_release(release):
     return release
 
 
-def platform_name(sys_platform, release):
+def platform_name(sys_platform, release=None):
     if not sys_platform:
         return "unknown"
     PLATFORMS = {"win32"    : "Microsoft Windows",
@@ -196,7 +183,7 @@ def platform_name(sys_platform, release):
                  }
     def rel(v):
         values = [v]
-        if type(release) in (tuple, list):
+        if isinstance(release, (tuple, list)):
             values += list(release)
         else:
             values.append(release)
@@ -210,7 +197,7 @@ def platform_name(sys_platform, release):
 
 def get_rand_chars(l=16, chars=b"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"):
     import random
-    return b"".join(chars[random.randint(0, len(chars)-1)] for _ in range(l))
+    return b"".join(chars[random.randint(0, len(chars)-1):][:1] for _ in range(l))
 
 def get_hex_uuid():
     return uuid.uuid4().hex
@@ -226,20 +213,22 @@ def get_machine_id():
     """
     v = u""
     if POSIX:
-        for filename in ["/etc/machine-id", "/var/lib/dbus/machine-id"]:
+        for filename in ("/etc/machine-id", "/var/lib/dbus/machine-id"):
             v = load_binary_file(filename)
             if v is not None:
                 break
-    return  str(v).strip("\n\r")
+    elif WIN32:
+        v = uuid.getnode()
+    return bytestostr(v).strip("\n\r")
 
 def get_user_uuid():
     """
         Try to generate a uuid string which is unique to this user.
         (relies on get_machine_id to uniquely identify a machine)
     """
-    uuid = os.environ.get("XPRA_USER_UUID")
-    if uuid:
-        return uuid
+    user_uuid = os.environ.get("XPRA_USER_UUID")
+    if user_uuid:
+        return user_uuid
     import hashlib
     u = hashlib.sha1()
     def uupdate(ustr):
@@ -250,45 +239,74 @@ def get_user_uuid():
         uupdate(str(os.getuid()))
         uupdate(u"/")
         uupdate(str(os.getgid()))
-    uupdate(os.environ.get("HOME", ""))
+    uupdate(os.path.expanduser("~/"))
     return u.hexdigest()
 
 
-monotonic_time = time.time
 try:
     from xpra.monotonic_time import _monotonic_time     #@UnresolvedImport
     assert _monotonic_time()>0
     monotonic_time = _monotonic_time
-except Exception as e:
-    pass
+except (ImportError, AssertionError):
+    monotonic_time = time.time
 
 
 def is_X11():
-    if PYTHON2:
-        return True
     if OSX or WIN32:
         return False
-    from xpra.x11.gtk3.gdk_bindings import is_X11_Display   #@UnresolvedImport
-    x11 = is_X11_Display()
-    return x11
+    if PYTHON2:
+        return True
+    try:
+        from xpra.x11.gtk3.gdk_bindings import is_X11_Display   #@UnresolvedImport
+        return is_X11_Display()
+    except ImportError:
+        get_util_logger().debug("failed to load x11 bindings", exc_info=True)
+        return True
 
 def is_Wayland():
-    return os.environ.get("WAYLAND_DISPLAY") or os.environ.get("XDG_SESSION_TYPE")=="wayland"
+    return os.environ.get("GDK_BACKEND", "")!="x11" and (
+        os.environ.get("WAYLAND_DISPLAY") or os.environ.get("XDG_SESSION_TYPE")=="wayland"
+        )
 
 
-def is_distribution_variant(variant=b"Debian", os_file="/etc/os-release"):
+def is_distribution_variant(variant=b"Debian"):
     if not POSIX:
         return False
     try:
-        if get_linux_distribution()[0]==variant:
-            return True
-    except:
+        v = load_os_release_file()
+        return any(l.find(variant)>=0 for l in v.splitlines() if l.startswith(b"NAME="))
+    except Exception:
         pass
     try:
-        v = load_binary_file(os_file)
-        return any(l.find(variant)>=0 for l in v.splitlines() if l.startswith(b"NAME="))
-    except:
-        return False
+        if variant==b"RedHat" and get_linux_distribution()[0].startswith(variant):
+            return True
+        if get_linux_distribution()[0]==variant:
+            return True
+    except Exception:
+        pass
+    return False
+
+def get_distribution_version_id():
+    if not POSIX:
+        return ""
+    try:
+        v = load_os_release_file()
+        for l in v.splitlines():
+            if l.startswith(b"VERSION_ID="):
+                return l.split(b"=", 1)[1]
+    except Exception:
+        pass
+    return ""
+
+os_release_file_data = False
+def load_os_release_file():
+    global os_release_file_data
+    if os_release_file_data is False:
+        try:
+            os_release_file_data = load_binary_file("/etc/os-release")
+        except (OSError, IOError):
+            os_release_file_data = None
+    return os_release_file_data
 
 def is_Ubuntu():
     return is_distribution_variant(b"Ubuntu")
@@ -322,13 +340,13 @@ def get_linux_distribution():
         cmd = ["lsb_release", "-a"]
         try:
             p = subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-            out, _ = p.communicate()
+            out = p.communicate()[0]
             assert p.returncode==0 and out
-        except:
+        except Exception:
             try:
                 from xpra.scripts.config import python_platform
                 _linux_distribution = python_platform.linux_distribution()
-            except:
+            except Exception:
                 _linux_distribution = ("unknown", "unknown", "unknown")
         else:
             d = {}
@@ -347,26 +365,69 @@ def getUbuntuVersion():
     if distro and len(distro)==3 and distro[0]=="Ubuntu":
         ur = distro[1]  #ie: "12.04"
         try:
-            rnum = [int(x) for x in ur.split(".")]  #ie: [12, 4]
-            return rnum
-        except:
+            return tuple(int(x) for x in ur.split("."))  #ie: (12, 4)
+        except ValueError:
             pass
-    return []
+    return ()
 
 def is_unity():
-    return os.environ.get("XDG_CURRENT_DESKTOP", "").lower().startswith("unity")
+    return os.environ.get("XDG_CURRENT_DESKTOP", "").lower().find("unity")>=0
+
+def is_gnome():
+    return os.environ.get("XDG_CURRENT_DESKTOP", "").lower().find("gnome")>=0
+
+def is_kde():
+    return os.environ.get("XDG_CURRENT_DESKTOP", "").lower().find("kde")>=0
+
+
+def is_WSL():
+    if not POSIX:
+        return False
+    r = None
+    for f in ("/proc/sys/kernel/osrelease", "/proc/version"):
+        r = load_binary_file(f)
+        if r:
+            break
+    return r is not None and r.find(b"Microsoft")>=0
+
 
 def get_generic_os_name():
     for k,v in {
-        "linux"     : "linux",
-        "darwin"    : "osx",
-        "win"       : "win32",
-        "freebsd"   : "freebsd",
-        }.items():
+            "linux"     : "linux",
+            "darwin"    : "osx",
+            "win"       : "win32",
+            "freebsd"   : "freebsd",
+            }.items():
         if sys.platform.startswith(k):
             return v
     return sys.platform
 
+def get_cpu_count():
+    #sensible default:
+    cpus = 2
+    try:
+        try:
+            #python3:
+            cpus = os.cpu_count()
+        except AttributeError:
+            try:
+                from cpuinfo import get_cpu_info
+                cpus = int(get_cpu_info().get("count"))
+            except Exception:
+                #python2:
+                import multiprocessing
+                cpus = multiprocessing.cpu_count()
+    except Exception:
+        pass
+    return cpus
+
+
+def filedata_nocrlf(filename):
+    v = load_binary_file(filename)
+    if v is None:
+        get_util_logger().error("failed to load '%s'", filename)
+        return None
+    return v.strip(b"\n\r")
 
 def load_binary_file(filename):
     if not os.path.exists(filename):
@@ -381,34 +442,51 @@ def load_binary_file(filename):
 
 #here so we can override it when needed
 def force_quit(status=1):
-    os._exit(status)
+    os._exit(status)  #pylint: disable=protected-access
+
+
+def register_SIGUSR_signals(idle_add):
+    if not os.name=="posix":
+        return
+    from xpra.util import dump_all_frames, dump_gc_frames
+    def sigusr1(*_args):
+        log = get_util_logger().info
+        log("SIGUSR1")
+        idle_add(dump_all_frames, log)
+    def sigusr2(*_args):
+        log = get_util_logger().info
+        log("SIGUSR2")
+        idle_add(dump_gc_frames, log)
+    signal.signal(signal.SIGUSR1, sigusr1)
+    signal.signal(signal.SIGUSR2, sigusr2)
 
 
 def livefds():
     live = set()
     try:
         MAXFD = os.sysconf("SC_OPEN_MAX")
-    except:
+    except ValueError:
         MAXFD = 256
     for fd in range(0, MAXFD):
         try:
             s = os.fstat(fd)
+        except Exception:
+            continue
+        else:
             if s:
                 live.add(fd)
-        except:
-            continue
     return live
 
-def close_fds(excluding=[0, 1, 2]):
+def close_fds(excluding=(0, 1, 2)):
     try:
         MAXFD = os.sysconf("SC_OPEN_MAX")
-    except:
+    except ValueError:
         MAXFD = 256
     for i in range(0, MAXFD):
         if i not in excluding:
             try:
                 os.close(i)
-            except:
+            except (IOError, OSError):
                 pass
 
 def get_all_fds():
@@ -428,7 +506,7 @@ def get_all_fds():
     sys.stderr.write("Uh-oh, can't close fds, please port me to your system...\n")
     return fds
 
-def close_all_fds(exceptions=[]):
+def close_all_fds(exceptions=()):
     for fd in get_all_fds():
         try:
             if fd not in exceptions:
@@ -440,27 +518,29 @@ def close_all_fds(exceptions=[]):
 
 def use_tty():
     from xpra.util import envbool
-    NOTTY = envbool("XPRA_NOTTY", False)
-    stdin = sys.stdin
-    return not os.environ.get("MSYSCON") and not NOTTY and stdin and stdin.isatty()
+    if envbool("XPRA_NOTTY", False):
+        return False
+    from xpra.platform.gui import use_stdin
+    return use_stdin()
 
 
-def shellsub(s, subs={}):
+def shellsub(s, subs=None):
     """ shell style string substitution using the dictionary given """
-    for var,value in subs.items():
-        s = s.replace("$%s" % var, str(value))
-        s = s.replace("${%s}" % var, str(value))
+    if subs:
+        for var,value in subs.items():
+            s = s.replace("$%s" % var, str(value))
+            s = s.replace("${%s}" % var, str(value))
     return s
 
 
-def osexpand(s, actual_username="", uid=0, gid=0, subs={}):
+def osexpand(s, actual_username="", uid=0, gid=0, subs=None):
     def expanduser(s):
         if actual_username and s.startswith("~/"):
             #replace "~/" with "~$actual_username/"
             return os.path.expanduser("~%s/%s" % (actual_username, s[2:]))
         return os.path.expanduser(s)
     from collections import OrderedDict
-    d = OrderedDict(subs)
+    d = OrderedDict(subs or {})
     d.update({
         "PID"   : os.getpid(),
         "HOME"  : expanduser("~/"),
@@ -474,10 +554,8 @@ def osexpand(s, actual_username="", uid=0, gid=0, subs={}):
             from xpra.platform.xposix.paths import get_runtime_dir
             d["XDG_RUNTIME_DIR"] = os.environ.get("XDG_RUNTIME_DIR", get_runtime_dir())
     if actual_username:
-        d.update({
-            "USERNAME"  : actual_username,
-            "USER"      : actual_username,
-            })
+        d["USERNAME"] = actual_username
+        d["USER"] = actual_username
     #first, expand the substitutions themselves,
     #as they may contain references to other variables:
     ssub = OrderedDict()
@@ -498,7 +576,8 @@ def path_permission_info(filename, ftype=None):
             if os.path.isdir(filename):
                 ftype = "directory"
         info.append("permissions on %s %s: %s" % (ftype, filename, oct(stat.S_IMODE(stat_info.st_mode))))
-        import pwd,grp      #@UnresolvedImport
+        import pwd
+        import grp      #@UnresolvedImport
         user = pwd.getpwuid(stat_info.st_uid)[0]
         group = grp.getgrgid(stat_info.st_gid)[0]
         info.append("ownership %s:%s" % (user, group))
@@ -561,6 +640,9 @@ class OSEnvContext(object):
 
 class FDChangeCaptureContext(object):
 
+    def __init__(self):
+        self.enter_fds = []
+        self.exit_fds = []
     def __enter__(self):
         self.enter_fds = get_all_fds()
     def __exit__(self, *_args):
@@ -587,12 +669,11 @@ class nomodule_context(object):
 
     def __init__(self, module_name):
         self.module_name = module_name
-        self.saved_module = sys.modules.get(module_name)
-        sys.modules[module_name] = None
     def __enter__(self):
-        pass
+        self.saved_module = sys.modules.get(self.module_name)
+        sys.modules[self.module_name] = None
     def __exit__(self, *_args):
-        if sys.modules.get(self.module_name) == None:
+        if sys.modules.get(self.module_name) is None:
             if self.saved_module is None:
                 try:
                     del sys.modules[self.module_name]
@@ -606,7 +687,7 @@ class nomodule_context(object):
 class umask_context(object):
 
     def __init__(self, umask):
-        self.umask = umask        
+        self.umask = umask
     def __enter__(self):
         self.orig_umask = os.umask(self.umask)
     def __exit__(self, *_args):
@@ -628,8 +709,8 @@ def setbinarymode(fd):
         #turn on binary mode:
         try:
             import msvcrt
-            msvcrt.setmode(fd, os.O_BINARY)         #@UndefinedVariable
-        except:
+            msvcrt.setmode(fd, os.O_BINARY)         #@UndefinedVariable pylint: disable=no-member
+        except (OSError, IOError):
             get_util_logger().error("setting stdin to binary mode failed", exc_info=True)
 
 def find_lib_ldconfig(libname):
@@ -652,7 +733,7 @@ def find_lib_ldconfig(libname):
     p = subprocess.Popen([ldconfig, "-p"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
     data = bytestostr(p.communicate()[0])
 
-    libpath = re.search(pattern, data, re.MULTILINE)
+    libpath = re.search(pattern, data, re.MULTILINE)        #@UndefinedVariable
     if libpath:
         libpath = libpath.group(1)
     return libpath
@@ -734,7 +815,7 @@ def setuidgid(uid, gid):
             os.initgroups(username, gid)
         else:
             import grp      #@UnresolvedImport
-            groups = [gr.gr_gid for gr in grp.getgrall() if (username in gr.gr_mem)]
+            groups = [gr.gr_gid for gr in grp.getgrall() if username in gr.gr_mem]
             os.setgroups(groups)
     #change uid and gid:
     try:
@@ -769,7 +850,7 @@ def get_peercred(sock):
             pid, uid, gid = struct.unpack(b'3i',creds)
             log("peer: %s", (pid, uid, gid))
             return pid, uid, gid
-        except Exception as  e:
+        except IOError as  e:
             log("getsockopt", exc_info=True)
             log.error("Error getting peer credentials: %s", e)
             return None
